@@ -8,10 +8,10 @@
 # No external libraries such as ImageMagick are used here, this is a very lightweight solution to
 # finding image information.
 #
-# FastImage knows about GIF, JPEG, BMP, TIFF, PNG and PSD files.
+# FastImage knows about GIF, JPEG, BMP, TIFF, ICO, CUR, PNG, PSD and WEBP files.
 #
 # FastImage can also read files from the local filesystem by supplying the path instead of a uri.
-# In this case FastImage uses the Addressable library to read the file in chunks of 256 bytes until
+# In this case FastImage reads the file in chunks of 256 bytes until
 # it has enough. This is possibly a useful bandwidth-saving feature if the file is on a network
 # attached disk rather than truly local.
 #
@@ -36,7 +36,7 @@
 # * http://snippets.dzone.com/posts/show/805
 # * http://www.anttikupila.com/flash/getting-jpg-dimensions-with-as3-without-loading-the-entire-file/
 # * http://pennysmalls.wordpress.com/2008/08/19/find-jpeg-dimensions-fast-in-pure-ruby-no-ima/
-# * http://imagesize.rubyforge.org/
+# * https://rubygems.org/gems/imagesize
 # * https://github.com/remvee/exifr
 #
 
@@ -45,6 +45,7 @@ require 'addressable/uri'
 require 'fastimage/fbr.rb'
 require 'delegate'
 require 'pathname'
+require 'zlib'
 
 class FastImage
   attr_reader :size, :type
@@ -53,8 +54,6 @@ class FastImage
 
   class FastImageException < StandardError # :nodoc:
   end
-  class MoreCharsNeeded < FastImageException # :nodoc:
-  end
   class UnknownImageType < FastImageException # :nodoc:
   end
   class ImageFetchFailure < FastImageException # :nodoc:
@@ -64,9 +63,9 @@ class FastImage
   class CannotParseImage < FastImageException # :nodoc:
   end
 
-  DefaultTimeout = 2
+  DefaultTimeout = 2 unless const_defined?(:DefaultTimeout)
 
-  LocalFileChunkSize = 256
+  LocalFileChunkSize = 256 unless const_defined?(:LocalFileChunkSize)
 
   # Returns an array containing the width and height of the image.
   # It will return nil if the image could not be fetched, or if the image type was not recognised.
@@ -77,7 +76,7 @@ class FastImage
   # If you wish FastImage to raise if it cannot size the image for any reason, then pass
   # :raise_on_failure => true in the options.
   #
-  # FastImage knows about GIF, JPEG, BMP, TIFF, PNG and PSD files.
+  # FastImage knows about GIF, JPEG, BMP, TIFF, ICO, CUR, PNG, PSD and WEBP files.
   #
   # === Example
   #
@@ -98,6 +97,8 @@ class FastImage
   #   FastImage.size("http://pennysmalls.com/does_not_exist", :raise_on_failure=>true)
   #   => raises FastImage::ImageFetchFailure
   #   FastImage.size("http://stephensykes.com/favicon.ico", :raise_on_failure=>true)
+  #   => [16, 16]
+  #   FastImage.size("http://stephensykes.com/images/squareBlue.icns", :raise_on_failure=>true)
   #   => raises FastImage::UnknownImageType
   #   FastImage.size("http://stephensykes.com/favicon.ico", :raise_on_failure=>true, :timeout=>0.01)
   #   => raises FastImage::ImageFetchFailure
@@ -137,7 +138,7 @@ class FastImage
   #   => :bmp
   #   FastImage.type("test/fixtures/test.jpg")
   #   => :jpeg
-  #   FastImage.type("http://pennysmalls.com/does_not_exist")
+  #   FastImage.type("http://stephensykes.com/does_not_exist")
   #   => nil
   #   File.open("/some/local/file.gif", "r") {|io| FastImage.type(io)}
   #   => :gif
@@ -159,6 +160,7 @@ class FastImage
   def initialize(uri, options={})
     @property = options[:type_only] ? :type : :size
     @timeout = options[:timeout] || DefaultTimeout
+    @proxy_url = options[:proxy]
     @uri = uri
 
     if uri.respond_to?(:read)
@@ -167,12 +169,12 @@ class FastImage
       begin
         @parsed_uri = Addressable::URI.parse(uri)
       rescue Addressable::URI::InvalidURIError
-        fetch_using_open_uri
+        fetch_using_file_open
       else
         if @parsed_uri.scheme == "http" || @parsed_uri.scheme == "https"
           fetch_using_http
         else
-          fetch_using_open_uri
+          fetch_using_file_open
         end
       end
     end
@@ -235,6 +237,21 @@ class FastImage
         end
       end
 
+      case res['content-encoding']
+      when 'deflate', 'gzip', 'x-gzip'
+        begin
+          gzip = Zlib::GzipReader.new(FiberStream.new(read_fiber))
+        rescue FiberError, Zlib::GzipFile::Error
+          raise CannotParseImage
+        end
+
+        read_fiber = Fiber.new do
+          while data = gzip.readline
+            Fiber.yield data
+          end
+        end
+      end
+
       parse_packets FiberStream.new(read_fiber)
 
       break  # needed to actively quit out of the fetch
@@ -243,7 +260,11 @@ class FastImage
 
   def proxy_uri
     begin
-      proxy = ENV['http_proxy'] && ENV['http_proxy'] != "" ? Addressable::URI.parse(ENV['http_proxy']) : nil
+      if @proxy_url
+        proxy = Addressable::URI.parse(@proxy_url)
+      else
+        proxy = ENV['http_proxy'] && ENV['http_proxy'] != "" ? Addressable::URI.parse(ENV['http_proxy']) : nil
+      end
     rescue Addressable::URI::InvalidURIError
       proxy = nil
     end
@@ -288,8 +309,8 @@ class FastImage
     parse_packets FiberStream.new(read_fiber)
   end
 
-  def fetch_using_open_uri
-    open(@uri) do |s|
+  def fetch_using_file_open
+    File.open(@uri) do |s|
       fetch_using_read(s)
     end
   end
@@ -377,11 +398,28 @@ class FastImage
       :tiff
     when '8B'
       :psd
+    when "\0\0"
+      # ico has either a 1 (for ico format) or 2 (for cursor) at offset 3
+      case @stream.peek(3).bytes.to_a.last
+      when 1 then :ico
+      when 2 then :cur
+      end
+    when "RI"
+      if @stream.peek(12)[8..11] == "WEBP"
+        :webp
+      else
+        raise UnknownImageType
+      end
     else
       raise UnknownImageType
     end
   end
 
+  def parse_size_for_ico
+    @stream.read(8)[6..7].unpack('CC').map{|byte| byte == 0 ? 256 : byte }
+  end
+  alias_method :parse_size_for_cur, :parse_size_for_ico
+
   def parse_size_for_gif
     @stream.read(11)[6..10].unpack('SS')
   end
@@ -446,6 +484,45 @@ class FastImage
     [result.first, result.last.abs]
   end
 
+  def parse_size_for_webp
+    vp8 = @stream.read(16)[12..15]
+    len = @stream.read(4).unpack("V")
+    case vp8
+    when "VP8 "
+      parse_size_vp8
+    when "VP8L"
+      parse_size_vp8l
+    when "VP8X"
+      parse_size_vp8x
+    else
+      nil
+    end
+  end
+
+  def parse_size_vp8
+    w, h = @stream.read(10).unpack("@6vv")
+    [w & 0x3fff, h & 0x3fff]
+  end
+
+  def parse_size_vp8l
+    @stream.read(1) # 0x2f
+    b1, b2, b3, b4 = @stream.read(4).bytes.to_a
+    [1 + (((b2 & 0x3f) << 8) | b1), 1 + (((b4 & 0xF) << 10) | (b3 << 2) | ((b2 & 0xC0) >> 6))]
+  end
+
+  def parse_size_vp8x
+    flags = @stream.read(4).unpack("C")[0]
+    b1, b2, b3, b4, b5, b6 = @stream.read(6).unpack("CCCCCC")
+    width, height = 1 + b1 + (b2 << 8) + (b3 << 16), 1 + b4 + (b5 << 8) + (b6 << 16)
+
+    if flags & 8 > 0 # exif
+      # parse exif for orientation
+      # TODO: find or create test images for this
+    end
+
+    return [width, height]
+  end
+
   class Exif # :nodoc:
     attr_reader :width, :height
     def initialize(stream)
