@@ -24,13 +24,22 @@ GoodFixtures = {
   "test.psd"=>[:psd, [17, 32]],
   "exif_orientation.jpg"=>[:jpeg, [600, 450]],
   "infinite.jpg"=>[:jpeg, [160,240]],
-  "orient_2.jpg"=>[:jpeg, [230,408]]
+  "orient_2.jpg"=>[:jpeg, [230,408]],
+  "favicon.ico" => [:ico, [16, 16]],
+  "man.ico" => [:ico, [48, 48]],
+  "test.cur" => [:cur, [32, 32]],
+  "webp_vp8x.webp" => [:webp, [386, 395]],
+  "webp_vp8l.webp" => [:webp, [386, 395]],
+  "webp_vp8.webp" => [:webp, [550, 368]]
 }
 
 BadFixtures = [
   "faulty.jpg",
-  "test.ico"
+  "test_rgb.ct"
 ]
+# man.ico courtesy of http://www.iconseeker.com/search-icon/artists-valley-sample/business-man-blue.html
+# test_rgb.ct courtesy of http://fileformats.archiveteam.org/wiki/Scitex_CT
+# test.cur courtesy of http://mimidestino.deviantart.com/art/Clash-Of-Clans-Dragon-Cursor-s-Punteros-489070897
 
 TestUrl = "http://example.nowhere/"
 
@@ -50,6 +59,12 @@ BadFixtures.each do |fn|
   FakeWeb.register_uri(:get, "#{TestUrl}#{fn}", :body => File.join(FixturePath, fn))
 end
 
+GzipTestImg = "gzipped.jpg"
+FakeWeb.register_uri(:get, "#{TestUrl}#{GzipTestImg}", :body => File.join(FixturePath, GzipTestImg), :content_encoding => "gzip")
+GzipTestImgTruncated = "truncated_gzipped.jpg"
+FakeWeb.register_uri(:get, "#{TestUrl}#{GzipTestImgTruncated}", :body => File.join(FixturePath, GzipTestImgTruncated), :content_encoding => "gzip")
+GzipTestImgSize = [970, 450]
+
 class FastImageTest < Test::Unit::TestCase
   def test_should_report_type_correctly
     GoodFixtures.each do |fn, info|
@@ -74,7 +89,7 @@ class FastImageTest < Test::Unit::TestCase
   end
 
   def test_should_return_nil_when_image_type_not_known
-    assert_nil FastImage.size(TestUrl + "test.ico")
+    assert_nil FastImage.size(TestUrl + "test_rgb.ct")
   end
 
   def test_should_return_nil_if_timeout_occurs
@@ -101,7 +116,7 @@ class FastImageTest < Test::Unit::TestCase
 
   def test_should_raise_when_asked_when_image_type_not_known
     assert_raises(FastImage::UnknownImageType) do
-      FastImage.size(TestUrl + "test.ico", :raise_on_failure=>true)
+      FastImage.size(TestUrl + "test_rgb.ct", :raise_on_failure=>true)
     end
   end
 
@@ -157,7 +172,7 @@ class FastImageTest < Test::Unit::TestCase
   end
 
   def test_should_return_nil_when_image_type_not_known_for_local_file
-    assert_nil FastImage.size(File.join(FixturePath, "test.ico"))
+    assert_nil FastImage.size(File.join(FixturePath, "test_rgb.ct"))
   end
 
   def test_should_raise_when_asked_to_when_size_cannot_be_found_for_local_file
@@ -230,11 +245,18 @@ class FastImageTest < Test::Unit::TestCase
     assert_equal actual_size, size
   end
 
+  def test_should_fetch_via_proxy_option
+    file = "test.gif"
+    actual_size = GoodFixtures[file][1]
+    size = FastImage.size(TestUrl + file, :proxy => "http://my.proxy.host:8080")
+    assert_equal actual_size, size
+  end
+
   def test_should_handle_https_image
     size = FastImage.size(HTTPSImage)
     assert_equal HTTPSImageInfo[1], size
   end
-  
+
   require 'pathname'
   def test_should_handle_pathname
     # bad.jpg does not have the size info in the first 256 bytes
@@ -243,4 +265,36 @@ class FastImageTest < Test::Unit::TestCase
     path = Pathname.new(File.join(FixturePath, "bad.jpg"))
     assert_equal([500,500], FastImage.size(path))
   end
+
+  def test_should_report_type_and_size_correctly_for_stringios
+    GoodFixtures.each do |fn, info|
+      string = File.read(File.join(FixturePath, fn))
+      stringio = StringIO.new(string)
+      assert_equal info[0], FastImage.type(stringio)
+      assert_equal info[1], FastImage.size(stringio)
+    end
+  end
+
+  def test_gzipped_file
+    url = "http://example.nowhere/#{GzipTestImg}"
+    assert_equal([970, 450], FastImage.size(url))
+  end
+
+  def test_truncated_gzipped_file
+    url = "http://example.nowhere/#{GzipTestImgTruncated}"
+    assert_raises(FastImage::SizeNotFound) do
+      FastImage.size(url, :raise_on_failure => true)
+    end
+  end
+
+  def test_cant_access_shell
+    url = "|echo>shell_test"
+    %x{rm -f shell_test}
+    FastImage.size(url)
+    assert_raises(Errno::ENOENT) do
+      File.open("shell_test")
+    end
+  ensure
+    %x{rm -f shell_test}
+  end
 end
